import { __read, __spread, __values } from "tslib";
import { groupBy } from '@antv/util';
export function partition(rows, groupby, orderby) {
    var sortFn = undefined;
    if (typeof orderby === 'function') {
        var key_1 = orderby;
        sortFn = function (left, right) {
            if (key_1(left) > key_1(right))
                return 1;
            if (key_1(left) < key_1(right))
                return -1;
            return 0;
        };
    }
    else {
        var keys_1 = typeof orderby === 'string' ? [orderby] : orderby;
        if (Array.isArray(keys_1) && keys_1.length > 0) {
            sortFn = function (left, right) {
                var e_1, _a;
                try {
                    for (var keys_2 = __values(keys_1), keys_2_1 = keys_2.next(); !keys_2_1.done; keys_2_1 = keys_2.next()) {
                        var field = keys_2_1.value;
                        if (left[field] > right[field])
                            return 1;
                        if (left[field] < right[field])
                            return -1;
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (keys_2_1 && !keys_2_1.done && (_a = keys_2.return)) _a.call(keys_2);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
                return 0;
            };
        }
    }
    var newRows = sortFn ? __spread(rows).sort(sortFn) : __spread(rows);
    var groupingFn = function () { return '_'; };
    if (typeof groupby === 'function') {
        groupingFn = groupby;
    }
    else if (Array.isArray(groupby) && groupby.length > 0) {
        groupingFn = function (row) { return "_" + groupby.map(function (col) { return row[col]; }).join('-'); };
        // NOTE: Object.keys({'b': 'b', '2': '2', '1': '1', 'a': 'a'}) => [ '1', '2', 'b', 'a' ]
        // that is why we have to add a prefix
    }
    else if (typeof groupby === 'string') {
        groupingFn = function (row) { return "_" + row[groupby]; };
    }
    return groupBy(newRows, groupingFn);
}
